//+------------------------------------------------------------------+
//|                                                      lotcalc.mqh |
//|                                      Copyright 2017, nicholishen |
//|                         https://www.forexfactory.com/nicholishen |
//+------------------------------------------------------------------+
#property copyright "Copyright 2017, nicholishen"
#property link      "https://www.forexfactory.com/nicholishen"
#property strict
#include "multi_symbol_timeframe.mqh"
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

double LotCalc(string symbol, double entry_price, double stoploss_price, double risk_percent = 2.0, double min_margin_level = 200.0)
{
   risk_percent     /= 100;
   min_margin_level /= 100;
// standard risk based lot calc
   double sym_min_lot         = SymbolInfoDouble(symbol, SYMBOL_VOLUME_MIN);
   double sym_max_lot         = SymbolInfoDouble(symbol, SYMBOL_VOLUME_MAX);
   double sym_lot_step        = SymbolInfoDouble(symbol, SYMBOL_VOLUME_STEP);
   double sym_tick_val        = TickValue(symbol);
          sym_tick_val        = sym_tick_val <= 0.0 ? 1.0 : sym_tick_val;
   double sym_tick_size       = SymbolInfoDouble(symbol, SYMBOL_TRADE_TICK_SIZE);
   double acc_balance         = AccountInfoDouble(ACCOUNT_BALANCE);
   double amount_to_risk      = acc_balance * risk_percent;
   int    num_ticks           = (int)NormalizeDouble(fabs(entry_price - stoploss_price) / sym_tick_size, 0);
   double move_val            = num_ticks * sym_tick_val;
   if(move_val == 0.0)
      return 0.0;
   double lots_standard       = amount_to_risk / move_val;
          lots_standard       = round(lots_standard / sym_lot_step) * sym_lot_step;
// calc max-lots_standard based on min margin_level after trade
   double margin_used         = AccountInfoDouble(ACCOUNT_MARGIN);
   double margin_available    = NormalizeDouble((acc_balance / min_margin_level) - margin_used, 2);
   double margin_req_per_lot  = MarginRequired(symbol);
   double lots_margin_level   = margin_available / margin_req_per_lot;
          lots_margin_level   = round(lots_margin_level / sym_lot_step) * sym_lot_step;
// calc max-lots_standard based on stop=loss creating a margin call @ the current account equity
   double acc_equity          = AccountInfoDouble(ACCOUNT_EQUITY);
   double margin_call         = AccountInfoDouble(ACCOUNT_MARGIN_SO_CALL) / 100.0;
   double lots_margin_call    = acc_equity/((margin_call * margin_req_per_lot ) + move_val);
          lots_margin_call    = round(lots_margin_call / sym_lot_step) * sym_lot_step;
   
   double res = fmin(fmin(lots_standard, lots_margin_level), lots_margin_call);
   if(res < sym_min_lot)
   {
      Print(__FUNCTION__, " NOT ENOUGH MONEY TO TRADE!");
      return 0.0;
   }
   return res > sym_max_lot ? sym_max_lot : res;
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

double MarginRequired(string symbol)
{
   string base          = StringSubstr(symbol, 0, 3);
   string counter       = StringSubstr(symbol, 3, 3);
   string account       = AccountInfoString(ACCOUNT_CURRENCY);
   double contract_size = SymbolInfoDouble(symbol, SYMBOL_TRADE_CONTRACT_SIZE);
   double leverage      = (double)AccountInfoInteger(ACCOUNT_LEVERAGE);
   if(base == account)
      return ceil(contract_size / leverage * 100.0) / 100.0;
   if(counter == account)
      return ceil(contract_size * iBid(symbol) / leverage * 100.0) /  100.0;
   string suffix  = StringSubstr(symbol, 6);
   for(int i = SymbolsTotal(false) - 1; i >= 0; i--)
   {
      string _symbol = SymbolName(i, false);
      if((suffix == "" || StringFind(_symbol, suffix) >= 0) && StringFind(_symbol, base) >= 0 && StringFind(_symbol, account) >= 0)
      {
         string _base    = StringSubstr(_symbol, 0, 3);
         string _counter = StringSubstr(_symbol, 3, 3);
         double _price   = iBid(_symbol);
         if(_base == account)
            return ceil(1 / _price * contract_size / leverage * 100.0) / 100.0;
         if(_counter == account)
            return ceil(_price * contract_size / leverage * 100.0) / 100.0;
         break;
      }
   }
   return 100000.0 / leverage;
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
enum BAR_PRICE_MODE {OPEN, HIGH, LOW, CLOSE};
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

double TickValue(string symbol = NULL, int timeframe = PERIOD_CURRENT, BAR_PRICE_MODE mode = CLOSE, int shift = 0)
{
   symbol     = symbol == NULL ? _Symbol : symbol;
   double point      = SymbolInfoDouble(symbol, SYMBOL_POINT);
   string base       = StringSubstr(symbol, 0, 3);
   string counter    = StringSubstr(symbol, 3, 3);
   string account    = AccountInfoString(ACCOUNT_CURRENCY);
   double tick_factor = NormalizeDouble(SymbolInfoDouble(symbol, SYMBOL_TRADE_TICK_SIZE) / point, 0);
   double contract_size = SymbolInfoDouble(symbol, SYMBOL_TRADE_CONTRACT_SIZE);
   double unit_factor = NormalizeDouble(contract_size * point, 2);
   if(counter == account)
      return unit_factor * tick_factor;
   if(base == account)
   {
      double p = GetPrice(symbol, timeframe, mode, shift);
      if(p <= 0.0)
         return SymbolInfoDouble(symbol, SYMBOL_TRADE_TICK_VALUE);
      return unit_factor / p * tick_factor;
   }
   string suffix     = StringSubstr(symbol, 6);
   for(int i = SymbolsTotal(false) - 1; i >= 0; i--)
   {
      string _symbol = SymbolName(i, false);
      if((suffix == "" || StringFind(_symbol, suffix) >= 0) && StringFind(_symbol, counter) >= 0 && StringFind(_symbol, account) >= 0)
      {
         string _base    = StringSubstr(_symbol, 0, 3);
         string _counter = StringSubstr(_symbol, 3, 3);
         double _price   = GetPrice(_symbol, timeframe, mode, shift);
         if(_price <= 0.0)
            return SymbolInfoDouble(symbol, SYMBOL_TRADE_TICK_VALUE);
         if(_base == account)
            return unit_factor / _price * tick_factor;
         if(_counter == account)
            return unit_factor / (1 / _price) * tick_factor;
         break;
      }
   }
   return 0.0;
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

double GetPrice(string symbol, int timeframe, BAR_PRICE_MODE mode, int shift)
{
   timeframe = timeframe == PERIOD_CURRENT ? _Period : timeframe;
   double bid = iBid(symbol);
   double res = 0.0;
   switch(mode)
   {
      case OPEN:
         if(!iOpen(res, symbol, timeframe, shift))
            return bid;
         else
            return res;
      case HIGH:
         if(!iHigh(res, symbol, timeframe, shift))
            return bid;
         else
            return res;
      case LOW:
         if(!iLow(res, symbol, timeframe, shift))
            return bid;
         else
            return res;
      case CLOSE:
         if(!iClose(res, symbol, timeframe, shift))
            return bid;
         else
            return res;
   }
   return bid;
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

